//
// Animations for Mandala
//

// Include files

#include <stdio.h>

#include "time.h"
#include "tlc5947.h"
#include "zxgesture.h"
#include "animations.h"

// Global variables

/* Values for each LED sets */
unsigned int groupes[NB_GROUPES];

/* Definition of LED sets */
int groupe1[]={
  23,10,7,40,61,70,55,13,-1
  };
int groupe2[]={
  5,9,39,43,63,48,57,20,-1
  };
int groupe3[]={
  22,24,8,29,30,33,38,42,60,64,67,49,52,58,14,18,-1
  };
int groupe4[]={
  0,4,6,26,31,34,37,44,47,65,68,50,53,59,15,19,-1

  };
int groupe5[]={
  2,11,25,27,32,35,41,45,62,66,71,51,56,12,17,21,-1
};
int groupe6[]={
  1,3,28,36,46,69,54,16,-1
};

int tous[]={
  23,10,7,40,61,70,55,13,
  5,9,39,43,63,48,57,20,
  22,24,8,29,30,33,38,42,60,64,67,49,52,58,14,18,
  0,4,6,26,31,34,37,44,47,65,68,50,53,59,15,19,
  2,11,25,27,32,35,41,45,62,66,71,51,56,12,17,21,
  1,3,28,36,46,69,54,16,
  -1
  };

/* Animations1 */
variation_params v1={.deltac=500,.deltad=500,.deltap=2000,.min=0,.max=1024};
variation_params v2={.deltac=500,.deltad=500,.deltap=2000,.min=0,.max=1024};
variation_params v3={.deltac=500,.deltad=500,.deltap=2000,.min=0,.max=256};
variation_params v4={.deltac=500,.deltad=500,.deltap=2000,.min=0,.max=1024};
variation_params v5={.deltac=500,.deltad=500,.deltap=2000,.min=0,.max=1024};
variation_params v6={.deltac=500,.deltad=500,.deltap=2000,.min=0,.max=1024};
animation a1[]={
  {0,60000,groupe1,&v1,variation_groupes},
  {500,60000,groupe2,&v2,variation_groupes},
  {1000,60000,groupe3,&v3,variation_groupes},
  {1500,60000,groupe4,&v4,variation_groupes},
  {2000,60000,groupe5,&v5,variation_groupes},
  {2500,60000,groupe6,&v6,variation_groupes},
  {-1,-1,NULL,NULL,NULL}
  };

/* Animation2 */
variation_params v_tous={.deltac=100,.deltad=100,.deltap=0,.min=0,.max=255};
animation a2[]={
  {0,40000,tous,&v_tous,variation_groupe},
  {-1,-1,NULL,NULL,NULL}
  };

/* Animation3 */
variation_params v21={.deltac=500,.deltad=500,.deltap=0,.min=0,.max=1024};
variation_params v22={.deltac=500,.deltad=500,.deltap=0,.min=0,.max=1024};
variation_params v23={.deltac=500,.deltad=500,.deltap=0,.min=0,.max=256};
variation_params v24={.deltac=500,.deltad=500,.deltap=0,.min=0,.max=1024};
variation_params v25={.deltac=500,.deltad=500,.deltap=0,.min=0,.max=1024};
variation_params v26={.deltac=500,.deltad=500,.deltap=0,.min=0,.max=1024};
animation a3[]={
  {0,60000,groupe1,&v21,variation_groupes},
  {0,60000,groupe2,&v22,variation_groupes},
  {0,60000,groupe3,&v23,variation_groupes},
  {0,60000,groupe4,&v24,variation_groupes},
  {0,60000,groupe5,&v25,variation_groupes},
  {0,60000,groupe6,&v26,variation_groupes},
  {-1,-1,NULL,NULL,NULL}
  };
 
animation *anims[]={a1,a2,a3,NULL};

/* Variables for animation settings */ 
int quantum=1000;
int stop;

// Functions
//
/* Handle the user input */

void user_input(void){
if(zx_gesture_ready()){
  int g=zx_get_gesture();
  printf("geste=%d quantum=%d\n",g,quantum);
  switch(g){
    case ZX_GEST_RIGHT:
      if(quantum>QUANTUM_MIN) quantum -= QUANTUM_PAS;
      break;
    case ZX_GEST_LEFT:
      if(quantum<QUANTUM_MAX) quantum += QUANTUM_PAS;
      break;
    case ZX_GEST_UP:
      stop=1;
      break;
    }
  }
}

/* Wait some time */

void wait_us(unsigned long delay){
static unsigned long start;
if(delay==0) start=micros();
else
  while(1){
    unsigned long now=micros();
    unsigned long delta=now-start;
    user_input();
    if(delta>=delay) break;
    }
}

/* Get a set of LEDs size */

int taille_groupes(int leds[]){
int i=0;
while(leds[i]>=0) i++;
return i;
}

/* Set luminosity value to a set of LEDs */

void valeur_groupes(int leds[],unsigned int valeur){
int i=0;
while(1){ if(leds[i]<0) break; groupes[leds[i++]]=valeur; }
}

/* Compute variation of luminosity for a set of LEDs */

void variation_groupes(int leds[],void *arg,long temps){
variation_params *p=arg;
long periode=p->deltac+p->deltad+p->deltap;	// Dans quelle phase de la variation se trouve-t-on ?
long instant=temps%periode;
if(instant<=p->deltac){				// Cas de la phase de croissance de p->min à p->max
  long calc=(long)p->max-(long)p->min;
  calc=calc*instant/p->deltac+p->min;
  valeur_groupes(leds,calc);
  } 
else if(instant<=p->deltac+p->deltad){		// Cas de la phase de décroissance
  long calc=(long)p->min-(long)p->max;
  calc=calc*(instant-p->deltac)/p->deltad+p->max;
  valeur_groupes(leds,calc);
  }
else 						// Cas de la pause
  valeur_groupes(leds,p->min);
}

/* Variation set by set */

void variation_groupe(int leds[],void *arg,long temps){
variation_params *p=arg;
long periode=p->deltac+p->deltad+p->deltap;
long nb=temps/periode;				// Quel groupe de LED est concerné ?
nb=nb%taille_groupes(leds);
int instant=temps%periode;			// Quelle est la phase courante de la variation ?
if(instant<=p->deltac){				// Cas de la phase de croissance de p->min à p->max
  long calc=(long)p->max-(long)p->min; calc=calc*instant/p->deltac+p->min;
  groupes[leds[nb]]=calc;
  }
else if(instant<=p->deltac+p->deltad){		// Cas de la phase de décroissance
  long calc=(long)p->min-(long)p->max; calc=calc*(instant-p->deltac)/p->deltad+p->max;
  groupes[leds[nb]]=calc;
  }
else 						// Cas de la pause
  groupes[leds[nb]]=p->min;
}

/* Play an animation */

void animation_play(animation *a){
static unsigned char init;
if(init==0){ init_LED_Drivers(NB_DRIVERS); init_time(); init=1; }
animation *p;
unsigned long temps=0;
int i;
for(i=0;i<NB_GROUPES;i++) groupes[i]=0;
while(stop==0){
  unsigned char actif=0;
  wait_us(0);
  for(p=a;p->depart>=0;p++){
    user_input();
    if(temps>p->arret) continue;
    actif=1;
    if(temps<p->depart) continue;
    p->fun_anim(p->groupes,p->params,temps-p->depart);
    }
  if(!actif) break;
  set_LED_Drivers(groupes,NB_DRIVERS);
  wait_us(quantum);
  temps++;
  }
stop=0;
}

/* Play all animations */

void animation_all(void){
printf("Play all\n");
animation **a=anims;
while(*a!=NULL) animation_play(*a++);
}
